# -*- coding: utf-8 -*-
from Products.PortalTransforms.libtransforms.piltransform import PILTransforms


class image_to_pcx(PILTransforms):
    __name__ = "image_to_pcx"
    inputs = ('image/*', )
    output = 'image/pcx'
    format = 'pcx'


def register():
    return image_to_pcx()



# -*- coding: utf-8 -*-
from Products.CMFPlone.utils import safe_unicode
from Products.PortalTransforms.utils import log

import os
import sys
import warnings


try:
    # Need to be imported before win32api to avoid dll loading
    # problems.
    import pywintypes
    import pythoncom

    import win32api
    WIN32 = True
except ImportError:
    WIN32 = False


class MissingBinary(Exception):
    pass

envPath = os.environ['PATH']
bin_search_path = [path for path in envPath.split(os.pathsep)
                   if os.path.isdir(path)]

cygwin = 'c:/cygwin'

# cygwin support
if sys.platform == 'win32' and os.path.isdir(cygwin):
    for p in ['/bin', '/usr/bin', '/usr/local/bin']:
        p = os.path.join(cygwin, p)
        if os.path.isdir(p):
            bin_search_path.append(p)

if sys.platform == 'win32':
    extensions = ('.exe', '.com', '.bat', )
else:
    extensions = ()


def bin_search(binary):
    """search the bin_search_path for a given binary returning its fullname or
       raises MissingBinary"""
    mode = os.R_OK | os.X_OK
    for path in bin_search_path:
        for ext in ('', ) + extensions:
            pathbin = os.path.join(path, binary) + ext
            if os.access(pathbin, mode) == 1:
                return pathbin

    raise MissingBinary('Unable to find binary "%s" in %s' %
                        (binary, os.pathsep.join(bin_search_path)))


def getShortPathName(binary):
    if WIN32:
        try:
            binary = win32api.GetShortPathName(binary)
        except win32api.error:
            log("Failed to GetShortPathName for '%s'" % binary)
    return binary


def sansext(path):
    return os.path.splitext(os.path.basename(path))[0]


##########################################################################
# The code below is taken from CMFDefault.utils to remove
# dependencies for Python-only installations
##########################################################################

def bodyfinder(text):
    """ Return body or unchanged text if no body tags found.

    Always use html_headcheck() first.
    Accepts bytes or text. Returns text.
    """
    text = safe_unicode(text)
    lowertext = text.lower()
    bodystart = lowertext.find('<body')
    if bodystart == -1:
        return text
    bodystart = lowertext.find('>', bodystart) + 1
    if bodystart == 0:
        return text
    bodyend = lowertext.rfind('</body>', bodystart)
    if bodyend == -1:
        return text
    return text[bodystart:bodyend]


def scrubHTMLNoRaise(html):
    """ Strip illegal HTML tags from string text.  """
    warnings.warn(("Call to deprecated function `scrubHTMLNoRaise` or `scrubHTML`."
                   "Use SafeHTML().scrub_html(html) instead."),
                  category=DeprecationWarning,
                  stacklevel=2)
    from Products.PortalTransforms.transforms.safe_html import SafeHTML
    transform = SafeHTML()
    return transform.scrub_html(html)


# BBB This is not used in this module. Remove with Plone 6.0
scrubHTML = scrubHTMLNoRaise



# -*- coding: utf-8 -*-
# Need to be imported first to avoid dll loading problems.
import pythoncom
import pywintypes

from Products.PortalTransforms.libtransforms.commandtransform import commandtransform  # noqa
from Products.PortalTransforms.libtransforms.utils import bodyfinder
from Products.PortalTransforms.transforms.safe_html import SafeHTML
from win32com.client import Dispatch
from win32com.client import constants
from win32com.client import gencache

import os
import os.path
import win32api
import win32com
import win32com.client


class document(commandtransform):

    def __init__(self, name, data):
        """Initialization: create tmp work
        directory and copy the document into a file"""
        commandtransform.__init__(self, name)
        name = self.name()
        if not name.endswith('.doc'):
            name = name + ".doc"
        self.tmpdir, self.fullname = self.initialize_tmpdir(data,
                                                            filename=name)

    def convert(self):
        try:
            # initialize COM for multi-threading, ignoring any errors
            # when someone else has already initialized differently.
            pythoncom.CoInitializeEx(pythoncom.COINIT_MULTITHREADED)
        except pythoncom.com_error:
            pass

        word = Dispatch("Word.Application")
        word.Visible = 0
        word.DisplayAlerts = 0
        doc = word.Documents.Open(self.fullname)
        # Let's set up some html saving options for this document
        doc.WebOptions.RelyOnCSS = 1
        doc.WebOptions.OptimizeForBrowser = 1
        doc.WebOptions.BrowserLevel = 0  # constants.wdBrowserLevelV4
        doc.WebOptions.OrganizeInFolder = 0
        doc.WebOptions.UseLongFileNames = 1
        doc.WebOptions.RelyOnVML = 0
        doc.WebOptions.AllowPNG = 1
        # And then save the document into HTML
        doc.SaveAs(FileName="%s.htm" % (self.fullname),
                   FileFormat=8)  # constants.wdFormatHTML)

        # TODO -- Extract Metadata (author, title, keywords) so we
        # can populate the dublin core
        # Converter will need to be extended to return a dict of
        # possible MD fields

        doc.Close()
        # word.Quit()

    def html(self):
        htmlfile = open(self.fullname + '.htm', 'r')
        html = htmlfile.read()
        htmlfile.close()
        html = SafeHTML().scrub_html(html)
        body = bodyfinder(html)
        return body

# This function has to be done. It's more difficult to delete the temp
# directory under Windows, because there is sometimes a directory in it.
# def cleanDir(self, tmpdir):



# -*- coding: utf-8 -*-
"""
A custom transform using external command
"""

from os import system
from os.path import dirname
from os.path import exists
from os.path import join
from Products.PortalTransforms.interfaces import ITransform
from Products.PortalTransforms.libtransforms.commandtransform import commandtransform  # noqa
from Products.PortalTransforms.libtransforms.utils import bin_search
from Products.PortalTransforms.libtransforms.utils import sansext
from Products.PortalTransforms.utils import log
from zope.interface import implementer

import re


from six.moves import cStringIO as StringIO


@implementer(ITransform)
class XsltTransform(commandtransform):
    """ Custom external command

    transform xml content by launching an external XSLT processor

    Input and output mime types must be set correctly !

    You can associate different document type to different transformations.
    """

    __name__ = "xml_to_html"

    def __init__(self, name=None, **kwargs):
        self.config = {
            # sample configuration
            'binary_path': bin_search('xsltproc'),
            'command_line': '%(transform)s %(input)s',
            'inputs': ('text/xml',),
            'output': 'text/html',
            'output_encoding': 'UTF-8',
            'dtds': {
                '-//OASIS//DTD DocBook V4.1//EN':
                    '/usr/share/sgml/docbook/xsl-stylesheets-1.29/html/'
                    'docbook.xsl',
            },
            'default_transform': '',
        }
        self.config_metadata = {
            'binary_path': (
                'string',
                'Binary path',
                'Path of the executable on the server.'),
            'command_line': (
                'string',
                'Command line',
                '''Additional command line option.  There should be at
least the input file (designed by "%(input)s") and the xsl file
(designed by "%(transform)s"). The transformation\'s result must be
printed on stdout.
'''),
            'inputs': (
                'list',
                'Inputs',
                'Input(s) MIME type. Change with care.'),
            'output': (
                'string',
                'Output',
                'Output MIME type. Change with care.'),
            'output_encoding': (
                'string',
                'Output encoding',
                'Output encoding.'),
            'dtds': (
                'dict',
                'DTDs',
                'Association of public ids or dtds to XSL transformations.',
                ('Public id', 'XSLT path')),
            'default_transform': (
                'string',
                'Default xslt',
                'Default xslt, used when no specific transformation is '
                'found.'
            ),
        }
        self.config.update(kwargs)
        if name:
            self.__name__ = name

    def __getattr__(self, attr):
        if attr == 'inputs':
            return self.config['inputs']
        if attr == 'output':
            return self.config['output']
        if attr == 'output_encoding':
            return self.config['output_encoding']
        raise AttributeError(attr)

    def convert(self, data, cache, **kwargs):
        base_name = sansext(kwargs.get("filename") or 'unknown.xml')
        dtds = self.config['dtds']
        tmpdir, fullname = self.initialize_tmpdir(data, filename=base_name)
        try:
            try:
                doctype = get_doctype(data)
            except DTException:
                try:
                    doctype = get_dtd(data)
                except DTException:
                    log('Unable to get doctype nor dtd in %s' % data)
                    doctype = None
            if doctype and doctype in dtds:
                data = self.invokeCommand(fullname, dtds[doctype])
            elif self.config['default_transform']:
                data = self.invokeCommand(fullname,
                                          self.config['default_transform'])
            cache.setData(data)
            path, images = self.subObjects(tmpdir)
            objects = {}
            if images:
                self.fixImages(path, images, objects)
                cache.setSubObjects(objects)
            return cache
        finally:
            self.cleanDir(tmpdir)

    def invokeCommand(self, input_name, xsl):
        output_file = join(dirname(input_name), 'tr_output')
        command = '%(binary_path)s %(command_line)s' % self.config
        data = {'input': input_name, 'output': output_file, 'transform': xsl}
        system(command % data)

        if exists(output_file):
            data = open(output_file).read()
        else:
            data = 'error occurs during transform. See error log'
        return data


def register():
    return XsltTransform()

DT_RGX = re.compile('<!DOCTYPE \w* PUBLIC \"([^"]*)\" \"([^"]*)\"')
DT_RGX2 = re.compile('<!DOCTYPE \w* SYSTEM \"([^"]*)\"')


class DTException(Exception):
    pass


def get_doctype(data):
    """ return the public id for the doctype given some raw xml data
    """
    if not hasattr(data, 'readlines'):
        data = StringIO(data)
    for line in data.readlines():
        line = line.strip()
        if not line:
            continue
        if line.startswith('<?xml') or line.startswith('<!-- '):
            continue
        m = DT_RGX.match(line)
        if m is not None:
            return m.group(1)
        else:
            raise DTException('Unable to match doctype in "%s"' % line)


def get_dtd(data):
    """ return the public id for the doctype given some raw xml data
    """
    if not hasattr(data, 'readlines'):
        data = StringIO(data)
    for line in data.readlines():
        line = line.strip()
        if not line:
            continue
        if line.startswith('<?xml') or line.startswith('<!-- '):
            continue
        m = DT_RGX.match(line)
        if m is not None:
            return m.group(2)
        m = DT_RGX2.match(line)
        if m is not None:
            return m.group(1)
        else:
            raise DTException('Unable to match doctype in "%s"' % line)


if __name__ == '__main__':
    print(get_doctype('''<?xml version="1.0" encoding="iso-8859-1"?>
<!DOCTYPE article PUBLIC "-//LOGILAB/DTD DocBook V4.1.2-Based Extension V0.1//EN" "dcbk-logilab.dtd" []>

<book id="devtools_user_manual" lang="fr">
'''))



# -*- coding: utf-8 -*-
"""
Uses the http://www.freewisdom.org/projects/python-markdown/ module

Author: Tom Lazar <tom@tomster.org> at the archipelago sprint 2006
"""

from Products.PortalTransforms.interfaces import ITransform
from Products.PortalTransforms.utils import log
from zope.interface import implementer


import six


try:
    import markdown as markdown_transformer
except ImportError:
    HAS_MARKDOWN = False
    log('markdown_to_html: Could not import python-markdown.')
else:
    HAS_MARKDOWN = True


@implementer(ITransform)
class markdown(object):

    __name__ = "markdown_to_html"
    inputs = ("text/x-web-markdown",)
    output = "text/html"

    def __init__(self, name=None, enabled_extensions=('markdown.extensions.fenced_code', 'markdown.extensions.nl2br', ), **kwargs):
        self.config = {
            'enabled_extensions': enabled_extensions,
        }

        self.config_metadata = {
            'enabled_extensions': (
                'list',
                'enabled_extensions',
                'Look for available extensions at ' +
                'https://pythonhosted.org/Markdown/extensions/index.html ' +
                'or write your own.'
            ),
        }

        if name:
            self.__name__ = name

    def name(self):
        return self.__name__

    def __getattr__(self, attr):
        if attr in self.config:
            return self.config[attr]
        raise AttributeError(attr)

    def convert(self, orig, data, **kwargs):
        if HAS_MARKDOWN:
            # markdown expects unicode input:
            orig = six.text_type(orig.decode('utf-8'))
            # PortalTransforms, however expects a string as result,
            # so we encode the unicode result back to UTF8:
            html = markdown_transformer \
                .markdown(orig, extensions=self.config.get('enabled_extensions', [])) \
                .encode('utf-8')
        else:
            html = orig
        data.setData(html)
        return data


def register():
    return markdown()



# -*- coding: utf-8 -*-
from Products.CMFPlone.utils import safe_unicode
from Products.PortalTransforms.transforms.safe_html import html5entities
from os.path import abspath
from os.path import basename
from os.path import dirname
from os.path import join
from sys import modules
from unittest import TestSuite

import glob
import re
import six


def normalize_html(s):
    if six.PY3 and isinstance(s, six.binary_type):
        s = safe_unicode(s)
    s = re.sub(r"&nbsp;", " ", s)
    s = re.sub(r"&#160;", " ", s)
    s = re.sub(r"\s+", " ", s)
    s = re.sub(r"(?s)\s+<", "<", s)
    s = re.sub(r"(?s)>\s+", ">", s)
    s = re.sub(r"\r", "", s)
    s = re.sub(r"<A", "<a", s)
    return s


def html5entity(ent):
    mapped_ent = html5entities[ent]
    if six.PY2:
        mapped_ent = mapped_ent.encode('utf-8')
    return mapped_ent


def build_test_suite(package_name, module_names, required=1):
    """
    Utlitity for building a test suite from a package name
    and a list of modules.

    If required is false, then ImportErrors will simply result
    in that module's tests not being added to the returned
    suite.
    """

    suite = TestSuite()
    try:
        for name in module_names:
            the_name = package_name + '.' + name
            __import__(the_name, globals(), locals())
            suite.addTest(modules[the_name].test_suite())
    except ImportError:
        if required:
            raise
    return suite


PREFIX = abspath(dirname(__file__))


def input_file_path(file):
    return join(PREFIX, 'input', file)


def output_file_path(file):
    return join(PREFIX, 'output', file)


def read_file_data(path, mode='rb'):
    data = None
    with open(path, mode) as fd:
        data = fd.read()
    return data


def matching_inputs(pattern):
    return [basename(path) for path in
            glob.glob(join(PREFIX, "input", pattern))]


def load(dotted_name, globals=None):
    """ load a python module from it's name """
    mod = __import__(dotted_name, globals)
    components = dotted_name.split('.')
    for comp in components[1:]:
        mod = getattr(mod, comp)
    return mod



# -*- coding: utf-8 -*-
"""
Uses the xpdf (www.foolabs.com/xpdf)
"""

from Products.PortalTransforms.interfaces import ITransform
from Products.PortalTransforms.libtransforms.commandtransform import popentransform  # noqa
from zope.interface import implementer


@implementer(ITransform)
class pdf_to_text(popentransform):

    __name__ = "pdf_to_text"
    inputs = ('application/pdf',)
    output = 'text/plain'
    output_encoding = 'utf-8'

    binaryName = "pdftotext"
    binaryArgs = "- -enc UTF-8 -"


def register():
    return pdf_to_text()



# -*- coding: utf-8 -*-
from Products.PortalTransforms.libtransforms.retransform import retransform


import six
from six.moves import html_entities


class html_to_text(retransform):
    inputs = ('text/html',)
    output = 'text/plain'


def register():
    def sub_func(matchobj):
        full = matchobj.group()
        ent = matchobj.group(1)
        result = html_entities.name2codepoint.get(ent)
        if result is None:
            if ent.startswith('#'):
                res = six.unichr(int(ent[1:]))
            else:
                res = full
        else:
            res = six.unichr(result)

        if isinstance(full, six.text_type):
            return res
        return res.encode('utf-8')

    return html_to_text(
        "html_to_text",
        (r'<script [^>]>.*</script>(?im)', ' '),
        (r'<style [^>]>.*</style>(?im)', ' '),
        (r'<head [^>]>.*</head>(?im)', ' '),
        (r'(?im)</?(font|em|i|strong|b)(?=\W)[^>]*>', ''),
        (r'<[^>]*>(?i)(?m)', ' '),
        (r'&([a-zA-Z0-9#]*?);', sub_func),
    )



# -*- coding: utf-8 -*-
from plone.app.contenttypes.testing import PLONE_APP_CONTENTTYPES_FIXTURE
from plone.app.testing import PloneSandboxLayer
from plone.app.testing.layers import FunctionalTesting
from plone.app.testing.layers import IntegrationTesting
from zope.configuration import xmlconfig


class ProductsPortalTransformsLayer(PloneSandboxLayer):

    defaultBases = (PLONE_APP_CONTENTTYPES_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        import Products.PortalTransforms
        xmlconfig.file(
            'configure.zcml',
            Products.PortalTransforms,
            context=configurationContext
        )


PRODUCTS_PORTALTRANSFORMS_FIXTURE = ProductsPortalTransformsLayer()

PRODUCTS_PORTALTRANSFORMS_INTEGRATION_TESTING = IntegrationTesting(
    bases=(PRODUCTS_PORTALTRANSFORMS_FIXTURE,),
    name="PortalTransformsLayer:Integration"
)
PRODUCTS_PORTALTRANSFORMS_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(PRODUCTS_PORTALTRANSFORMS_FIXTURE,),
    name="PortalTransformsLayer:Functional"
)



# -*- coding: utf-8 -*-
import unittest

from Products.CMFPlone.utils import safe_unicode
from Products.PortalTransforms.testing import PRODUCTS_PORTALTRANSFORMS_INTEGRATION_TESTING  # noqa


class TransformTestCase(unittest.TestCase):

    layer = PRODUCTS_PORTALTRANSFORMS_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.transforms = self.portal.portal_transforms

    def _baseAssertEqual(self, first, second, msg=None):
        return unittest.TestCase._baseAssertEqual(
            self, safe_unicode(first), safe_unicode(second), msg)

    def assertMultiLineEqual(self, first, second, msg=None):
        return unittest.TestCase.assertMultiLineEqual(
            self, safe_unicode(first), safe_unicode(second), msg)



# -*- coding: utf8  -*-
from __future__ import print_function
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.interfaces import IFilterSchema
from Products.CMFPlone.utils import safe_unicode
from Products.PortalTransforms.data import datastream
from Products.PortalTransforms.interfaces import IDataStream
from Products.PortalTransforms.libtransforms.utils import MissingBinary
from Products.PortalTransforms.tests.base import TransformTestCase
from Products.PortalTransforms.tests.utils import html5entity
from Products.PortalTransforms.tests.utils import input_file_path
from Products.PortalTransforms.tests.utils import load
from Products.PortalTransforms.tests.utils import matching_inputs
from Products.PortalTransforms.tests.utils import normalize_html
from Products.PortalTransforms.tests.utils import output_file_path
from Products.PortalTransforms.tests.utils import read_file_data
from Products.PortalTransforms.transforms.image_to_bmp import image_to_bmp
from Products.PortalTransforms.transforms.image_to_gif import image_to_gif
from Products.PortalTransforms.transforms.image_to_jpeg import image_to_jpeg
from Products.PortalTransforms.transforms.image_to_pcx import image_to_pcx
from Products.PortalTransforms.transforms.image_to_png import image_to_png
from Products.PortalTransforms.transforms.image_to_ppm import image_to_ppm
from Products.PortalTransforms.transforms.image_to_tiff import image_to_tiff
from Products.PortalTransforms.transforms.markdown_to_html import HAS_MARKDOWN
from Products.PortalTransforms.transforms.safe_html import SafeHTML
from Products.PortalTransforms.transforms.textile_to_html import HAS_TEXTILE
from os.path import exists
from plone.registry.interfaces import IRegistry
from xml.sax.saxutils import unescape
from zope.component import getUtility

import hashlib
import itertools
import os
import six


# we have to set locale because lynx output is locale sensitive !
os.environ['LC_ALL'] = 'C'


class TransformTest(TransformTestCase):

    def setUp(self):
        super(TransformTest, self).setUp()
        self.request = self.layer['request']
        registry = getUtility(IRegistry)
        self.settings = registry.forInterface(IFilterSchema, prefix="plone")

    def md5(self, fname):
        hash_md5 = hashlib.md5()
        with open(fname, "rb") as f:
            for chunk in iter(lambda: f.read(4096), b""):
                hash_md5.update(chunk)
        return hash_md5.hexdigest()

    def do_convert(self, filename=None):
        if filename is None and exists(self.output + '.nofilename'):
            output = self.output + '.nofilename'
        else:
            output = self.output
        orig = read_file_data(self.input)
        data = datastream(self.transform.name())
        res_data = self.transform.convert(orig, data, filename=filename)
        self.assertTrue(IDataStream.providedBy(res_data))
        got = res_data.getData()
        expected = ''
        try:
            expected = read_file_data(self.output, 'rb')
        except IOError:
            import sys
            print('No output file found.', file=sys.stderr)
            print(
                'File {0} created, check it !'.format(self.output),
                file=sys.stderr)
            with open(output, 'wb') as fd:
                fd.write(got)
            self.assertTrue(0)

        if six.PY3 and isinstance(expected, six.binary_type):
            got = safe_unicode(got)
            expected = safe_unicode(expected)

        if self.normalize is not None:
            got = self.normalize(got)
            expected = self.normalize(expected)

        # show the first character ord table for debugging
        got_start = got.strip()[:20]
        expected_start = expected.strip()[:20]
        msg = 'IN {0}({1}) expected:\n{2}\nbut got:\n{3}'.format(
            self.transform.name(),
            self.input,
            "%s %s" % (expected_start, str([ord(x) for x in expected_start])),
            "%s %s" % (got_start, str([ord(x) for x in got_start])),
        )

        # compare md5 sum of the whole file content
        self.assertEqual(
            got_start,
            expected_start,
            msg,
        )
        self.assertEqual(
            self.subobjects,
            len(res_data.getSubObjects()),
            '%s\n\n!=\n\n%s\n\nIN %s(%s)' % (
                self.subobjects,
                len(res_data.getSubObjects()),
                self.transform.name(),
                self.input,
            )
        )

    def testSame(self):
        try:
            self.do_convert(filename=self.input)
        except MissingBinary:
            pass

    def testSameNoFilename(self):
        try:
            self.do_convert()
        except MissingBinary:
            pass

    def __repr__(self):
        return self.transform.name()


class PILTransformsTest(TransformTestCase):

    def setUp(self):
        super(PILTransformsTest, self).setUp()
        self.request = self.layer['request']
        self.mimetypes_registry = getToolByName(self.portal,
                                                'mimetypes_registry')

    def guess_mimetype(self, data):
        return str(self.mimetypes_registry.classify(data))

    def assert_image_transformed(self, factory, filename, input_mimetype, target_mimetype):
        self.transforms.registerTransform(factory)
        data = read_file_data(input_file_path(filename))
        self.assertEqual(self.guess_mimetype(data), input_mimetype)
        data = self.transforms.convertTo(target_mimetype=target_mimetype, orig=data)
        self.assertEqual(data.getMetadata()['mimetype'], target_mimetype)

    def test_image_to_bmp(self):
        self.assert_image_transformed(
            image_to_bmp(), 'logo.jpg', 'image/jpeg', 'image/x-ms-bmp')

    def test_image_to_gif(self):
        self.assert_image_transformed(
            image_to_gif(), 'logo.png', 'image/png', 'image/gif')

    def test_image_to_jpeg(self):
        self.assert_image_transformed(
            image_to_jpeg(), 'logo.gif', 'image/gif', 'image/jpeg')

    def test_image_to_png(self):
        self.assert_image_transformed(
            image_to_png(), 'logo.jpg', 'image/jpeg', 'image/png')

    def test_image_to_pcx(self):
        self.assert_image_transformed(
            image_to_pcx(), 'logo.gif', 'image/gif', 'image/pcx')

    def test_image_to_ppm(self):
        self.assert_image_transformed(
            image_to_ppm(), 'logo.png', 'image/png', 'image/x-portable-pixmap')

    def test_image_to_tiff(self):
        self.assert_image_transformed(
            image_to_tiff(), 'logo.jpg', 'image/jpeg', 'image/tiff')


class SafeHtmlTransformsTest(TransformTestCase):

    def setUp(self):
        super(SafeHtmlTransformsTest, self).setUp()
        self.request = self.layer['request']
        registry = getUtility(IRegistry)
        self.settings = registry.forInterface(
            IFilterSchema, prefix="plone")
        self.settings.valid_tags.append('style')
        self.settings.valid_tags.remove('h1')
        self.settings.nasty_tags.append('h1')

    def tearDown(self):
        self.settings.valid_tags.remove('style')
        self.settings.valid_tags.append('h1')

    def test_kill_nasty_tags_which_are_not_valid(self):
        self.assertTrue('script' in self.settings.nasty_tags)
        self.assertFalse('script' in self.settings.valid_tags)
        orig = '<p><script>foo</script></p>'
        data_out = '<p/>'
        data = self.transforms.convertTo(target_mimetype='text/x-html-safe', orig=orig)
        self.assertEqual(data.getData(), data_out)

        self.assertTrue('h1' in self.settings.nasty_tags)
        self.assertFalse('h1' in self.settings.valid_tags)
        orig = '<p><h1>foo</h1></p>'
        data_out = '<p/>'
        data = self.transforms.convertTo(target_mimetype='text/x-html-safe', orig=orig)
        self.assertEqual(data.getData(), data_out)

    def test_entityiref_attributes(self):
        orig = '<a href="&uuml;">foo</a>'
        data_out = '<a href="&#xFC;">foo</a>'
        data = self.transforms.convertTo(target_mimetype='text/x-html-safe', orig=orig)
        self.assertEqual(data.getData(), data_out)

    def test_charref_attributes(self):
        orig = '<a href="&#0109;">foo</a>'
        data_out = '<a href="m">foo</a>'
        data = self.transforms.convertTo(target_mimetype='text/x-html-safe', orig=orig)
        self.assertEqual(data.getData(), data_out)

    def test_entityiref_data(self):
        orig = '<p>foo &uuml; bar</p>'
        data_out = '<p>foo {} bar</p>'.format(html5entity('uuml;'))
        data = self.transforms.convertTo(target_mimetype='text/x-html-safe', orig=orig)
        self.assertEqual(data.getData(), data_out)

    def test_charref_data(self):
        orig = '<p>bar &#0109; foo</p>'
        data_out = '<p>bar m foo</p>'
        data = self.transforms.convertTo(target_mimetype='text/x-html-safe', orig=orig)
        self.assertEqual(data.getData(), data_out)


class SafeHtmlTransformsWithScriptTest(TransformTestCase):

    def setUp(self):
        super(SafeHtmlTransformsWithScriptTest, self).setUp()
        self.request = self.layer['request']
        registry = getUtility(IRegistry)
        self.settings = registry.forInterface(
            IFilterSchema, prefix="plone")
        self.settings.valid_tags.append('script')
        self.settings.nasty_tags.remove('script')

    def tearDown(self):
        self.settings.nasty_tags.append('script')
        self.settings.valid_tags.remove('script')

    def test_entities_outside_script(self):
        orig = "<code>a > 0 && b < 1</code>"
        escaped = '<code>a &gt; 0 &amp;&amp; b &lt; 1</code>'
        data = self.transforms.convertTo(target_mimetype='text/x-html-safe', orig=orig)
        self.assertEqual(data.getData(), escaped)

    def test_script_and_entities_and_unicode(self):
        _all = (
           ''
           # script with not converted entity
           '<script type="text/javascript">$("h1 > ul").hide();</script>',
           # script with not converted entity and unicode
           '<script type="text/javascript">'
           '$("h1 > ul").attr("alt", "Officiële");</script>',
           # script
           '<script type="text/javascript">var el = "test";</script>',
           # entity
           '<p>(KU&nbsp;Loket)</p>',
           # unicode
           '<p>Officiële inschrijvingen </p>',
        )
        nbsp = html5entity('nbsp;')
        for tokens in itertools.product(_all, repeat=5):
            orig = ''.join(tokens)
            data = self.transforms.convertTo(
                target_mimetype='text/x-html-safe',
                orig=orig
            )
            self.assertEqual(
                unescape(data.getData()),
                orig.replace('&nbsp;', nbsp))

    def test_script_with_all_entities_and_unicode(self):
        orig = ('<p>Officiële inschrijvingen</p>',
                '<script type="text/javascript">'
                '$("h1 > ul").hide();'
                'entities = "&copy;";'
                '</script>',
                '<p>(KU&nbsp;Loket)</p>',
                )
        escd = ('<p>Officiële inschrijvingen</p>',
                '<script type="text/javascript">'
                '$("h1 > ul").hide();'
                'entities = "&copy;";'
                '</script>',
                '<p>(KU{}Loket)</p>'.format(html5entity('nbsp;')),
                )

        _all = six.moves.zip(orig, escd)
        for tokens in itertools.product(_all, repeat=4):
            orig_tokens, escaped_tokens = zip(*tokens)
            orig = ''.join(orig_tokens)
            escaped = ''.join(escaped_tokens)
            data = self.transforms.convertTo(
                target_mimetype='text/x-html-safe',
                orig=orig
            )

            self.assertEqual(unescape(data.getData()), escaped)


class WordTransformsTest(TransformTestCase):

    def setUp(self):
        super(WordTransformsTest, self).setUp()
        self.request = self.layer['request']
        registry = getUtility(IRegistry)
        self.settings = registry.forInterface(
            IFilterSchema, prefix="plone")

    def test_ignore_javascript_attrs(self):
        data = read_file_data(input_file_path('test_js.doc'))
        # should not throw exception even though it holds javascript link
        self.transforms.convertTo(target_mimetype='text/html', orig=data)


class ParsersTestCase(TransformTestCase):

    def setUp(self):
        super(ParsersTestCase, self).setUp()
        self.request = self.layer['request']
        registry = getUtility(IRegistry)
        self.settings = registry.forInterface(
            IFilterSchema, prefix="plone")

    def test_javascript_on_attr(self):
        data = read_file_data(input_file_path('test_js_on.html'))
        result = SafeHTML().scrub_html(data)
        self.assertTrue('link' in result)

    def test_javascript_uri(self):
        data = read_file_data(input_file_path('test_js_uri.html'))
        result = SafeHTML().scrub_html(data)
        self.assertTrue('link' in result)

    def test_invalid_tags(self):
        data = read_file_data(input_file_path('test_invalid_tags.html'))
        self.assertEqual(SafeHTML().scrub_html(data).strip(), '')


TRANSFORMS_TESTINFO = (
    ('Products.PortalTransforms.transforms.pdf_to_html',
     "demo1.pdf", "demo1.html", normalize_html, 0,
     ),
    ('Products.PortalTransforms.transforms.word_to_html',
     "test.doc", "test_word.html", normalize_html, 0,
     ),
    ('Products.PortalTransforms.transforms.lynx_dump',
     "test_lynx.html", "test_lynx.txt", None, 0,
     ),
    ('Products.PortalTransforms.transforms.html_to_text',
     "test_lynx.html", "test_html_to_text.txt", None, 0,
     ),
    ('Products.PortalTransforms.transforms.identity',
     "rest1.rst", "rest1.rst", None, 0,
     ),
    ('Products.PortalTransforms.transforms.text_to_html',
     "rest1.rst", "rest1.html", None, 0,
     ),
    ('Products.PortalTransforms.transforms.safe_html',
     "test_safehtml.html", "test_safe.html", None, 0,
     ),
    ('Products.PortalTransforms.transforms.image_to_bmp',
     "logo.jpg", "logo.bmp", None, 0,
     ),
    ('Products.PortalTransforms.transforms.image_to_gif',
     "logo.bmp", "logo.gif", None, 0,
     ),
    ('Products.PortalTransforms.transforms.image_to_jpeg',
     "logo.gif", "logo.jpg", None, 0,
     ),
    ('Products.PortalTransforms.transforms.image_to_png',
     "logo.bmp", "logo.png", None, 0,
     ),
    ('Products.PortalTransforms.transforms.image_to_ppm',
     "logo.gif", "logo.ppm", None, 0,
     ),
    ('Products.PortalTransforms.transforms.image_to_tiff',
     "logo.png", "logo.tiff", None, 0,
     ),
    ('Products.PortalTransforms.transforms.image_to_pcx',
     "logo.png", "logo.pcx", None, 0,
     ),
)
if HAS_MARKDOWN:
    TRANSFORMS_TESTINFO = TRANSFORMS_TESTINFO + (
        ('Products.PortalTransforms.transforms.markdown_to_html',
         "markdown.txt", "markdown.html", None, 0,
         ),
    )
if HAS_TEXTILE:
    TRANSFORMS_TESTINFO = TRANSFORMS_TESTINFO + (
        ('Products.PortalTransforms.transforms.textile_to_html',
         "input.textile", "textile.html", None, 0,
         ),
    )


def initialise(transform, normalize, pattern):
    global TRANSFORMS_TESTINFO
    for fname in matching_inputs(pattern):
        outname = '%s.out' % fname.split('.')[0]
        TRANSFORMS_TESTINFO += ((transform, fname, outname, normalize, 0),)


# ReST test cases
initialise('Products.PortalTransforms.transforms.rest', normalize_html,
           "rest*.rst")
# Python test cases
initialise('Products.PortalTransforms.transforms.python', normalize_html,
           "*.py")

# FIXME missing tests for image_to_html, st

TR_NAMES = None


def make_tests(test_descr=TRANSFORMS_TESTINFO):
    """generate tests classes from test info

    return the list of generated test classes
    """
    tests = []
    for _transform, tr_input, tr_output, _normalize, _subobjects in test_descr:
        # load transform if necessary
        if isinstance(_transform, type('')):
            try:
                _transform = load(_transform).register()
            except MissingBinary:
                # we are not interessted in tests with missing binaries
                continue
            except Exception:
                import traceback
                traceback.print_exc()
                continue

        if TR_NAMES is not None and not _transform.name() in TR_NAMES:
            print('skip test for {0}'.format(_transform.name()))
            continue

        class TransformTestSubclass(TransformTest):
            input = input_file_path(tr_input)
            output = output_file_path(tr_output)
            transform = _transform
            normalize = lambda x, y: _normalize(y)  # noqa
            subobjects = _subobjects

        tests.append(TransformTestSubclass)

    tests.append(PILTransformsTest)
    tests.append(SafeHtmlTransformsTest)
    tests.append(SafeHtmlTransformsWithScriptTest)
    tests.append(WordTransformsTest)
    tests.append(ParsersTestCase)
    return tests


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    for test in make_tests():
        suite.addTest(makeSuite(test))
    return suite



"""
PortalTransforms setup handlers.
"""
from __future__ import print_function
from Products.CMFCore.utils import getToolByName

from six import StringIO


def correctMapping(out, portal):
    pt = getToolByName(portal, 'portal_transforms')
    pt_ids = pt.objectIds()

    for m_in, m_out_dict in pt._mtmap.items():
        for m_out, transforms in m_out_dict.items():
            for transform in transforms:
                if transform.id not in pt_ids:
                    # error, mapped transform is no object in
                    # portal_transforms. correct it!
                    print(
                        'have to unmap transform ({0}) cause its not in '
                        'portal_transforms ...'.format(transform.id), file=out)
                    try:
                        pt._unmapTransform(transform)
                    except:
                        raise
                    else:
                        print('...ok', file=out)


def updateTransform(out, portal, transform_id):
    print('Update {0}...'.format(transform_id), file=out)
    transform_module = "Products.PortalTransforms.transforms.{0}".format(transform_id)
    pt = getToolByName(portal, 'portal_transforms')
    for item in pt.objectIds():
        transform = getattr(pt, item)
        if transform.id == transform_id and \
                transform.module == transform_module:
            try:
                transform.get_parameter_value('disable_transform')
            except KeyError:
                out.write('  replace {0} ({1}, {2}) ...'.format(
                    item, transform.name(), transform.module))
                try:
                    pt.unregisterTransform(item)
                    pt.manage_addTransform(item, transform_module)
                except:
                    raise
                else:
                    out.write('  ...done')

    out.write('  ...done')


def installPortalTransforms(portal):
    out = StringIO()

    updateTransform(out, portal, 'safe_html')
    updateTransform(out, portal, 'markdown_to_html')

    correctMapping(out, portal)


def setupPortalTransforms(context):
    """
    Setup PortalTransforms step.
    """
    # Only run step if a flag file is present (e.g. not an extension profile)
    if context.readDataFile('portal-transforms-various.txt') is None:
        return
    site = context.getSite()
    installPortalTransforms(site)



# -*- coding: utf-8 -*-
"""
Uses lynx -dump
"""
from Products.PortalTransforms.interfaces import ITransform
from Products.PortalTransforms.libtransforms.commandtransform import \
    popentransform
from zope.interface import implementer


@implementer(ITransform)
class lynx_dump(popentransform):

    __name__ = "lynx_dump"
    inputs = ('text/html',)
    output = 'text/plain'

    binaryName = "lynx"
    # XXX does -stdin work on windows?
    binaryArgs = "-dump -stdin -force_html"


def register():
    return lynx_dump()
